Package: EHR
Version: 0.4-11
Date: 2022-12-15
Title: Electronic Health Record (EHR) Data Processing and Analysis Tool
Maintainer: Leena Choi <leena.choi@vanderbilt.edu>
Authors@R: 
    c(person("Leena", "Choi", role = c("aut", "cre"),
             email = "leena.choi@vanderbilt.edu",
             comment = c(ORCID = "0000-0002-2544-7090")),
    person("Cole", "Beck", role = "aut",
           email = "cole.beck@vumc.org",
           comment = c(ORCID = "0000-0002-6849-6255")),
    person("Hannah", "Weeks", role = "aut",
           email = "hannah.l.weeks@vanderbilt.edu",
           comment = c(ORCID = "0000-0002-0262-6790")),
    person("Elizabeth", "McNeer", role = "aut",
           email = "elizabeth.mcneer@vumc.org"),
    person("Nathan", "James", role = "aut",
           email = "nathan.t.james@vanderbilt.edu",
           comment = c(ORCID = "0000-0001-7079-9151")),
    person("Michael", "Williams", role = "aut",
           email = "michael.l.williams@vanderbilt.edu"))
Description: Process and analyze electronic health record (EHR) data. The 'EHR'
    package provides modules to perform diverse medication-related studies using
    data from EHR databases. Especially, the package includes modules to perform
    pharmacokinetic/pharmacodynamic (PK/PD) analyses using EHRs, as outlined in
    Choi, Beck, McNeer, Weeks, Williams, James, Niu, Abou-Khalil, Birdwell, Roden, 
    Stein, Bejan, Denny, and Van Driest (2020) <doi:10.1002/cpt.1787>. Additional 
    modules will be added in future. In addition, this package provides various 
    functions useful to perform Phenome Wide Association Study (PheWAS) to explore 
    associations between drug exposure and phenotypes obtained from EHR data, as 
    outlined in Choi, Carroll, Beck, Mosley, Roden, Denny, and Van Driest (2018) 
    <doi:10.1093/bioinformatics/bty306>.
Depends: R (>= 2.10)
License: GPL (>= 3)
URL: https://choileena.github.io/
Imports: stats, utils, data.table, methods, lubridate, pkdata
Suggests: glmnet, logistf, medExtractR (>= 0.4.1), knitr, rmarkdown,
        ggplot2, markdown
NeedsCompilation: no
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Packaged: 2022-12-19 16:07:42 UTC; beckca
Author: Leena Choi [aut, cre] (<https://orcid.org/0000-0002-2544-7090>),
  Cole Beck [aut] (<https://orcid.org/0000-0002-6849-6255>),
  Hannah Weeks [aut] (<https://orcid.org/0000-0002-0262-6790>),
  Elizabeth McNeer [aut],
  Nathan James [aut] (<https://orcid.org/0000-0001-7079-9151>),
  Michael Williams [aut]
Repository: CRAN
Date/Publication: 2022-12-19 18:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 16:36:05 UTC; unix
